@@ -102,11 +102,19 @@ const char *profile2string(VAProfile profile)
         PROFILE(H264Main)
         PROFILE(H264High)
         PROFILE(H264ConstrainedBaseline)
+        PROFILE(H264MultiviewHigh)
+        PROFILE(H264StereoHigh)
         PROFILE(VC1Simple)
         PROFILE(VC1Main)
         PROFILE(VC1Advanced)
         PROFILE(JPEGBaseline)
         PROFILE(VP8Version0_3)
+        PROFILE(HEVCMain)
+        PROFILE(HEVCMain10)
+        PROFILE(VP9Profile0)
+        PROFILE(VP9Profile1)
+        PROFILE(VP9Profile2)
+        PROFILE(VP9Profile3)
     }
     ASSERT(0);
     return "Unknown";
@@ -125,6 +133,7 @@ const char *entrypoint2string(VAEntrypoint entrypoint)
         ENTRYPOINT(Deblocking)
         ENTRYPOINT(EncSlice)
         ENTRYPOINT(EncPicture)
+        ENTRYPOINT(EncSliceLP)
         ENTRYPOINT(VideoProc)
     }
     ASSERT(0);
